import time
from datetime import datetime

import comtypes

_ipaddress = "192.168.0.151"  # add ip address of LabSat here
_filename = "api_python_sample"  # this is the name of the file that we will record then playback

# Define constants for enums that the api defines
# External Ref enum values
ExternalReferenceCom_NO = 0
ExternalReferenceCom_NotSet = -1
ExternalReferenceCom_OCXO = 2
ExternalReferenceCom_YES = 1

YesNoNotSet_No = 0
YesNoNotSet_NotSet = -1
YesNoNotSet_Yes = 1

# Constellation enum values
Gps = "GPS"
Beidou = "BEIDOU"
Glonass = "GLONASS"

# Channel enum values
Channel1 = "CHANNEL1"
Channel2 = "CHANNEL2"
Channel3 = "CHANNEL3"
Channel4 = "CHANNEL4"

# Record mode enum values
ChannelDigitalRecordModeCom_NotSet = -1
ChannelDigitalRecordModeCom_OFF = 0
ChannelDigitalRecordModeCom_1PPS = 1
ChannelDigitalRecordModeCom_CAN1 = 2
ChannelDigitalRecordModeCom_CAN2 = 3
ChannelDigitalRecordModeCom_RS232 = 4
ChannelDigitalRecordModeCom_DIGI1 = 5
ChannelDigitalRecordModeCom_DIGI2 = 6

# Bandwidth enum values
BandwidthNotSet = -1
Bandwidth10 = 10
Bandwidth30 = 30
Bandwidth56 = 56

# Quantisation enum values
QuantisationNotSet = -1
Quantisation1 = 1
Quantisation2 = 2
Quantisation3 = 3


def RunSampleCode():
    try:
        import comtypes
        import comtypes.client as cc

        print("loading com types")

        # get the com labsat3wb com library
        tlb_id = comtypes.GUID("{43526893-e7f2-4b7b-b8c3-a784a1e7d239}")
        cc.GetModule((tlb_id, 1, 0))
        print("loaded tlb file")

        labsat = cc.CreateObject("Racelogic.LabSat3WBAPI.LabSatApiCom")

        labsat.SerialNumberOrIpAddress = _ipaddress

        if labsat is not None:
            print("LabSat3WB API loaded successfully")
        else:
            print("Error loading LabSat3WB API. Exiting.")
            return

        # list files in the current directory
        ListFiles(labsat)

        print("Configuring LabSat")

        # Configure Display
        ConfigDisplay(labsat, 20, 100)

        # Reset to default
        ConfigDisplay(labsat, 50, 50)

        # Configure Constellation
        ConfigConstellation(labsat, Bandwidth56, Quantisation3, 1607)
        time.sleep(0.1)
        ConfigConstellation(labsat, Bandwidth10, Quantisation2, 1765, 1300)
        time.sleep(0.1)
        ConfigConstellation(labsat, Bandwidth10, Quantisation2, 1150, 1602, 1765)

        # Set Channel 1 Digital Record mode to RS232
        ConfigDigitalRecordMode(labsat, Channel1, ChannelDigitalRecordModeCom_RS232)

        # Switch Channel 2 Digital Record mode to digital 2
        ConfigDigitalRecordMode(labsat, Channel2, ChannelDigitalRecordModeCom_DIGI2)

        # List all files
        ListFiles(labsat)

        # Record a file
        RecordFile(labsat, _filename, 0)

        # allow to record for 120 seconds
        time.sleep(120)

        StopRecording(labsat)

        time.sleep(3)

        # Play the file that we have just recorded
        StartPlayback(labsat, _filename)

        # Give the labsat a little time to start playback before setting attenuation/mute
        time.sleep(5)

        # add attenuation to all channels
        SetAttenuationLevel(labsat, 5)

        # set attenuation for ch1
        SetAttenuationLevelOnConstellation(labsat, 6, YesNoNotSet_Yes, YesNoNotSet_No, YesNoNotSet_No)

        # mute all
        MuteAll(labsat, YesNoNotSet_Yes)
        time.sleep(1)

        # un-mute all
        MuteAll(labsat, YesNoNotSet_No)
        time.sleep(1)

        # mute a single channel
        MuteConstellation(labsat, 1, YesNoNotSet_No, YesNoNotSet_Yes, YesNoNotSet_No)

        # allow to play for 10 seconds
        time.sleep(10)

        StopPlayback(labsat)

        # Give the labsat a little time to stop
        time.sleep(3)

        # clean-up - delete file that we recorded
        DeleteFile(labsat, _filename)

        # labsat.Media.Options.ListDirectoriesAndFiles = YesNoNotSet_Yes
        labsat.Media.Command.Run()
        outputtext = labsat.Media.Command.OutputText

        print("Finished")

    except ImportError as err:
        print(err)


def ListFiles(labsat):
    print("List files in the current directory")
    labsat.Media.Options.DeleteFileWithName = _filename

    labsat.Media.Options.Reset()
    labsat.Media.Options.ListDirectoriesAndFiles = YesNoNotSet_Yes
    labsat.Media.Command.Run()
    outputtext = labsat.Media.Command.OutputText

    print(outputtext)


def DeleteFile(labsat, filename):
    # clear out params for this command
    labsat.Media.Options.Reset()

    print("Delete " + _filename + " if it already exists")
    labsat.Media.Options.DeleteFileWithName = _filename
    labsat.Media.Command.Run()
    print(labsat.Media.Command.OutputText)


# Record with a specified filename and a set duration (in seconds)
def RecordFile(labsat, filename, duration=0):
    print("Record file: " + filename)
    labsat.RecordFile.Options.fileName = filename

    # only set record duration if the duration isn't 0
    if duration != 0:
        labsat.RecordFile.Options.RecordForSeconds = duration

    labsat.RecordFile.Command.Run()
    print(labsat.RecordFile.Command.OutputText)


# Stops the recording
def StopRecording(labsat):
    print("Stop recording " + _filename)
    labsat.RecordFile.Options.Reset()
    labsat.RecordFile.Options.Stop = YesNoNotSet_Yes
    labsat.RecordFile.Command.Run()
    print(labsat.RecordFile.Command.OutputText)


def StartPlayback(labsat, fileName, fromSeconds=0, playForSeconds=0):
    print("Playback File " + fileName)

    labsat.Config.ConfigPlay.Options.Loop = 1
    labsat.PlayFile.Options.fileName = fileName

    print("labsat.Config.ConfigPlay.Options.Loop = " + str(labsat.Config.ConfigPlay.Options.Loop))

    # fromSeconds - the position in the playback file to start from
    if fromSeconds != 0:
        labsat.PlayFile.Options.fromSeconds = fromSeconds

    # playForSeconds - how long to play the file for
    if playForSeconds != 0:
        labsat.PlayFile.Options.playForSeconds = playForSeconds

    labsat.PlayFile.Command.Run()
    print(labsat.PlayFile.Command.OutputText)


def StopPlayback(labsat):
    print("Stopping playback")

    labsat.PlayFile.Options.Reset()
    labsat.PlayFile.Options.Stop = YesNoNotSet_Yes

    labsat.PlayFile.Command.Run()
    print(labsat.PlayFile.Command.OutputText)


# Configures the brightness and contrast of the display
def ConfigDisplay(labsat, brightness=None, contrast=None):
    print("Configuring Display")
    if not brightness is None:
        labsat.Config.ConfigSetupDisplay.Options.Brightness = brightness
        labsat.Config.ConfigSetupDisplay.Options.SetBrightness = True

    if not contrast is None:
        labsat.Config.ConfigSetupDisplay.Options.Contrast = contrast
        labsat.Config.ConfigSetupDisplay.Options.SetContrast = True

    labsat.Config.ConfigSetupDisplay.Command.Run()
    print(labsat.Config.ConfigSetupDisplay.Command.OutputText)


# Manually configures the device time to the current date time
def ManuallyConfigTime(labsat):
    datetoset = datetime.utcnow()
    print("Set time to " + str(datetoset))
    labsat.Config.ConfigSetup.Options.UseUtcTime = YesNoNotSet_No
    labsat.Config.ConfigSetup.Options.SetDeviceTime = datetoset.time()
    labsat.Config.ConfigSetup.Command.Run()
    print(labsat.Config.ConfigSetup.Command.OutputText)


# Configures the device to use UTC time
def ConfigUTCTime(labsat):
    print("Set time to use UTC")
    # Set the date to the minimum date in Python, so that the API ignores the date time if previously set by this API session
    labsat.Config.ConfigSetup.Options.SetDeviceTime = time.gmtime(0)
    labsat.Config.ConfigSetup.Options.UseUtcTime = YesNoNotSet_Yes

    labsat.Config.ConfigSetup.Command.Run()
    print(labsat.Config.ConfigSetup.Command.OutputText)


# Configures the constellations enabled/disabled status.
def ConfigConstellation(labsat, bandwidth, quantisation, frequency, frequency2=0, frequency3=0):
    print("Configure constellations bandwidth:" + str(bandwidth) + " quantisation:" + str(
        quantisation) + " frequency1:" + str(frequency) + " frequency2:" + str(frequency) + " frequency3:" + str(
        frequency))
    # For each parameter true, false or nothing values can be specified
    labsat.Config.ConfigConstellation.Options.Bandwidth = bandwidth
    labsat.Config.ConfigConstellation.Options.Quantization = quantisation
    labsat.Config.ConfigConstellation.Options.Frequency1 = frequency
    labsat.Config.ConfigConstellation.Options.Frequency2 = frequency2
    labsat.Config.ConfigConstellation.Options.Frequency3 = frequency3

    labsat.Config.ConfigConstellation.Command.Run()

    time.sleep(0.5)
    print(labsat.Config.ConfigSetup.Command.OutputText)


# Set the channel digital record mode
def ConfigDigitalRecordMode(labsat, channel, channelDigitalRecordMode):
    labsat.Config.ConfigSetup.Options.Reset()
    print("Configure digital record mode")
    if channel == Channel1:
        labsat.Config.ConfigSetup.Options.Channel1DigitalRecordMode = channelDigitalRecordMode
    elif channel == Channel2:
        labsat.Config.ConfigSetup.Options.Channel2DigitalRecordMode = channelDigitalRecordMode
    elif channel == Channel3:
        labsat.Config.ConfigSetup.Options.Channel3DigitalRecordMode = channelDigitalRecordMode

    labsat.Config.ConfigSetup.Command.Run()
    print(labsat.Config.ConfigSetup.Command.OutputText)


# Enables or disables the live NMEA stream (Use during playback)
def ConfigNMEAStream(labsat, enabled):
    print("Configure NMEA stream to " + str(enabled))
    labsat.Monitor.Options.NmeaEnabled = enabled
    labsat.Monitor.Command.Run()
    print(labsat.Monitor.Command.OutputText)


# Applies attenuation level to all constellations (Use during playback)
def SetAttenuationLevel(labsat, level):
    print("Configure Attenuation level to " + str(level))
    labsat.Attenuation.Options.Reset()
    labsat.Attenuation.Options.AttenuationLevel = level

    # if we don't specify any channel to get attenuation then it is applied to all.
    labsat.Attenuation.Options.ApplyToCh1 = YesNoNotSet_No
    labsat.Attenuation.Options.ApplyToCh2 = YesNoNotSet_No
    labsat.Attenuation.Options.ApplyToCh3 = YesNoNotSet_No
    labsat.Attenuation.Command.Run()
    print(labsat.Attenuation.Command.OutputText)


# Sets attenuation level on specified constellations (Use during playback)
def SetAttenuationLevelOnConstellation(labsat, level, applyToCh1, applyToCh2=YesNoNotSet_NotSet,
                                       applyToCh3=YesNoNotSet_NotSet):
    print("Set Attenuation of " + str(level) + " to ch1: " + str(applyToCh1) + " ch2: " + str(
        applyToCh2) + " ch3: " + str(applyToCh3))
    labsat.Attenuation.Options.AttenuationLevel = level
    labsat.Attenuation.Options.ApplyToCh1 = applyToCh1
    labsat.Attenuation.Options.ApplyToCh2 = applyToCh2
    labsat.Attenuation.Options.ApplyToCh3 = applyToCh3

    labsat.Attenuation.Command.Run()
    print(labsat.Attenuation.Command.OutputText)


# Mutes or Unmutes all satelite constellations (Use during playback)
def MuteAll(labsat, muted):
    print("Set Mute: " + str(muted))
    labsat.Mute.Options.Mute = muted
    labsat.Mute.Command.Run()
    print(labsat.Mute.Command.OutputText)


# TODO: Bug with mute COM
# Mutes or Unmutes specified satelite constellations (Use during playback)
def MuteConstellation(labsat, muted, applyToCh1, applyToCh2, applyToCh3):
    print(
        "Set Muted: " + str(muted) + " to GPS: " + str(applyToCh1) + " GLONASS: " + str(applyToCh2) + " Beidou: " + str(
            applyToCh3))
    labsat.Mute.Options.Mute = muted
    labsat.Mute.Options.ApplyToCh1 = applyToCh1
    labsat.Mute.Options.ApplyToCh2 = applyToCh2
    labsat.Mute.Options.ApplyToCh3 = applyToCh3

    labsat.Mute.Command.Run()
    print(labsat.Mute.Command.OutputText)


# run the main method
RunSampleCode()
